/**
 * \addtogroup Cflash
 * @{
 */

/*****************************************************************************/
/**
 * \file Cflash.h
 * Sector level Compact Flash routines for the 8051 and Keil C51.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 21-02-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************/

#ifndef CFLASH_H
#define CFLASH_H

#include "REG51.h"
#include <intrins.h>


/*****************************************************************************
 *
 *                               Macro Definitions
 *
 *****************************************************************************/

/** 
 * Compact Flash device docking status macro.
 * \param -
 * \return \b true once the CF card is fully inserted into it's receptacle.
 * \hideinitializer
 */
#define CFLASH_DOCKED()  ~CD1


/*****************************************************************************
 *
 *                               Defined Constants
 *
 *****************************************************************************/

/** 
 * Microcontroller port assigned to communicate data with the Compact Flash
 * device.
 * Adjust \b CF_DATA_PORT to match hardware. Port bits 0 through 7 are 
 * hardcoded to CF data bits 0 through 7 respectively.
 * \hideinitializer
 */
#define CF_DATA_PORT     P0


/** 
 * Microcontroller port assigned to communicate addressing with the Compact
 * Flash device.
 * Adjust \b CF_ADDRRESS_PORT to match hardware. Port bits 0 through 2 are 
 * hardcoded to CF address bits 0 through 2 respectively.
 * \hideinitializer
 */
#define CF_ADDRRESS_PORT P2


/** 
 * Microcontroller port assigned to communicate control with the Compact
 * Flash device.
 * Adjust \b CF_CONTROL_PORT to match hardware. Port bits 3 through 7 are 
 * hardcoded to CF controls as follows:
 *     - bit 3 is OE
 *     - bit 4 is CE1
 *     - bit 5 is WE
 *     - bit 6 is CD1
 *     - bit 7 is RDY
 * \hideinitializer
 */
#define CF_CONTROL_PORT  P2


/** 
 * Compact Flash sector size.
 * Adjust \b CF_SECTOR_SIZE to match the sector size of the CF device
 * (typically 512).
 * \hideinitializer
 */
#define CF_SECTOR_SIZE  512


/*
 * CF card control bit assignments.
 */
sbit OE   = CF_CONTROL_PORT^3;
sbit CE1  = CF_CONTROL_PORT^4;   
sbit WE   = CF_CONTROL_PORT^5;
sbit CD1  = CF_CONTROL_PORT^6;
sbit RDY  = CF_CONTROL_PORT^7;  


/*
 * CF card register addresses.
 */
#define CF_DATA_REG     0x00    // ADDRESS OF DATA REGISTER
#define CF_ERROR_REG    0x01    // ADDRESS OF ERROR REGISTER
#define CF_FEATURES_REG 0x01    // ADDRESS OF FEATURES REGISTER
#define CF_SEC_CNT_REG  0x02    // ADDRESS OF SECTOR COUNT REGISTER
#define CF_SEC_NUM_REG  0x03    // ADDRESS OF SECTOR NUMBER REGISTER
#define CF_CYL_LO_REG   0x04    // ADDRESS OF LOW CYLINDER REGISTER
#define CF_CYL_HI_REG   0x05    // ADDRESS OF HIGH CYLINDER REGISTER
#define CF_HEAD_REG     0x06    // ADDRESS OF HEAD/DRIVE REGISTER
#define CF_STATUS_REG   0x07    // ADDRESS OF STATUS REGISTER
#define CF_COMMAND_REG  0x07    // ADDRESS OF COMMAND REGISTER


/*
 * CF command assignments.
 */
#define CF_IDENTIFY     0xEC
#define CF_WRITE_SEC    0x30
#define CF_READ_SEC     0x20
#define CF_LBA          0xE0



/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * Compact Flash module initialisation function.
 * Must be called once to set-up CF hardware prior to calling any sector level
 * flash routines.
 * \param -
 * \return -
 */
extern void cflash_init(void);


/** 
 * Compact Flash sector level read function.
 * Reads a CF sector, of length \b CF_SECTOR_SIZE and specified by the
 * given sector number, into a memory buffer referenced by the given pointer.
 * \param sector An \b unsigned \b long LBA sector number.
 * \param buffer An \b unsigned \b char \b pointer to \b xdata memory that
 *         will receive the read sector data.
 * \return -
 */
extern void cflash_rd_sector(unsigned long sector,
                             unsigned char xdata * buffer);


/** 
 * Compact Flash sector level write function.
 * Writes a CF sector, of length \b CF_SECTOR_SIZE and specified by the
 * given sector number, from a memory buffer referenced by the given pointer.
 * \param sector An \b unsigned \b long LBA sector number.
 * \param buffer An \b unsigned \b char \b pointer to \b xdata memory that
 *         holds the sector data to write.
 * \return -
 */
extern void cflash_wr_sector(unsigned long sector,
                             unsigned char xdata * buffer);


#endif

/** @} */
