/**
 * \addtogroup Dcfs16
 * @{
 */

/*****************************************************************************/
/**
 * \file Dcfs16.c
 * FAT16 DOS root directory file system for the 8051 and Keil C51.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 24-06-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#include "Dcfs16.h" 

/*****************************************************************************
 *
 *                                  Structures
 *
 *****************************************************************************/

/*
 * Structure of an MSDOS directory entry
 */
struct Dentry
{
    u8_t   Dname[11];                  /* Filename + extension               */
    u8_t   Dattr;                      /* File attributes                    */
    u8_t   Dreserved[10];              /* Reserved area                      */
    u16_t  Dtime;                      /* Time last modified                 */
    u16_t  Ddata;                      /* Date last modified                 */
    u16_t  Dcluster;                   /* First cluster number               */
    u16_t  Dsizel;                     /* File size (LOW)                    */
    u16_t  Dsizeh;                     /* File size (HIGH)                   */
};

/*
 * Structure of an internal file control block
 */
struct Fblock
{
    u8_t   Fattr;                      /* Open attributes                    */
    u8_t   Fsector;                    /* Sector within cluster              */
    struct Dentry    *Fdirptr;         /* Pointer to directory entry         */
    u16_t  FFirstDirectorySector;      /* Directory sector                   */
    u16_t  Ffirstcls;                  /* First cluster in file              */
    u16_t  Flastcls;                   /* Last cluster read/written          */
    u16_t  Fnextcls;                   /* Next cluster to read/write         */
    u16_t  Foffset;                    /* Read/Write offset                  */
    u16_t  Fsizel;                     /* File size (LOW)                    */
    u16_t  Fsizeh;                     /* File size (HIGH)                   */
};


/*****************************************************************************
 *
 *                          Global Variables Definitions  
 *
 *****************************************************************************/
/* 
 * Internal "work" sector variables
 */
static u16_t  wrksec = 4; //=0;?       /* Current work sector number         */
static bool_t wrkchg = DCFS16_FALSE;  /* Work sector changed                */
static u8_t xdata wrkbuff[DCFS16_SECTOR_SIZE];  /* Work sector buffer       */
static u8_t xdata Fbuffer[DCFS16_SECTOR_SIZE];  /* Data xfer buffer         */

/* 
 * Active drive information (other than contained in BPB) 
 */
static idata u16_t FirstDirectorySector; /* First sector of directory        */
static idata u16_t FirstDataSector;      /* First sector of data area        */
static idata u32_t CountofClusters;      /* Clusters in data region          */

/* 
 * Disk information (from BIOS Parameter Block) used globally 
 */
static idata u16_t  BPB_BytesPerSec;     /* Bytes per sector                 */
static idata u8_t   BPB_SecPerClus;      /* BPB_TotSec16 per cluster         */
static idata u16_t  BPB_RootEntCnt;      /* Num of directory entries         */
static idata u16_t  BPB_TotSec16;        /* BPB_TotSec16 on disk             */
static idata u16_t  BPB_FATSz16;         /* BPB_TotSec16 per fat             */
static idata u8_t   BPB_NumFATs;         /* Number of FAT tables             */

/* 
 * Open File Information
 */
static idata struct Fblock fp;


/*****************************************************************************
 *
 *                          Private Function Prototypes
 *
 *****************************************************************************/
static u16_t dcfs16_allocate(u16_t cluster);
static void dcfs16_release(u16_t cluster);
static u16_t dcfs16_get_fat(u16_t cluster);
static void dcfs16_set_fat(u16_t cluster, u16_t value);
static struct Dentry xdata *dcfs16_lookup(u8_t *file);
static struct Dentry xdata *dcfs16_create_file(u8_t *file, u8_t attrs);
static void dcfs16_parse_filename(u8_t buffer[], u8_t *name);
static void dcfs16_read_work(u16_t sector);
static void dcfs16_update_work(void);
static u16_t dcfs16_swap_endian_ui(u16_t fromVal);
static u32_t dcfs16_swap_endian_ul(u32_t fromVal);


/*****************************************************************************
 *
 *                         Public Function Implementation 
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                              dcfs16_mount_drive()
 *
 *****************************************************************************/
bool_t dcfs16_mount_drive(void)
{
    /*
     * Mount a disk drive and set up control information
     * THIS FUNCTION MUST BE CALLED BEFORE ACCESSING ANY FILES.
     */

    /* Disk information (from BIOS Parameter Block) used locally */
    u16_t  BPB_RsvdSecCnt;                /* # reserved BPB_TotSec16         */
    u32_t  BPB_FATSz32;                   /* Sectors in 1 FAT                */
    u32_t  BPB_TotSec32;                  /* BPB_TotSec32 on disk            */

    u16_t  RootDirSectors;                /* Count of root directory sectors */
    u16_t  FATSz;                         /* Fat Table size                  */

    /* Initialise sector level storage media hardware. */
    DCFS16_MEDIA_INIT();

    /* Check that sector level storage media exists */
    if (0 == DCFS16_MEDIA_DOCKED())
    {
        /* No media to mount */
        return DCFS16_FAIL;
    }

    /* Load BIOS Parameter Block into wrkbuff */
    dcfs16_read_work(DCFS16_BPB_SECTOR);

    /* Retrieve BPB values needed for calculations
       BPB is little endian, Keil variables are big endian. */
    BPB_BytesPerSec = dcfs16_swap_endian_ui(*((u16_t *)(wrkbuff+11)));
    BPB_SecPerClus  = *(wrkbuff + 13);
    BPB_RsvdSecCnt  = dcfs16_swap_endian_ui(*((u16_t *)(wrkbuff+14)));
    BPB_NumFATs     = *(wrkbuff + 16);
    BPB_RootEntCnt  = dcfs16_swap_endian_ui(*((u16_t *)(wrkbuff+17)));
    BPB_TotSec16    = dcfs16_swap_endian_ui(*((u16_t *)(wrkbuff+19)));
    BPB_FATSz16     = dcfs16_swap_endian_ui(*((u16_t *)(wrkbuff+22)));
    BPB_TotSec32    = dcfs16_swap_endian_ul(*((u32_t *)(wrkbuff+32)));
    BPB_FATSz32     = dcfs16_swap_endian_ul(*((u32_t *)(wrkbuff+36)));

    /* Calculate the count of sectors occupied by the root directory */
    RootDirSectors = ((BPB_RootEntCnt * 32u)
                                + (BPB_BytesPerSec - 1u)) / BPB_BytesPerSec;

    /* Calculate start of the data region, the first sector of cluster 2 */
    if(0u != BPB_FATSz16)
    {
        FATSz = BPB_FATSz16;
    }
    else
    {
        FATSz = BPB_FATSz32;
    }

    FirstDataSector = BPB_RsvdSecCnt + (BPB_NumFATs * FATSz) + RootDirSectors;

    /* Calculate start of the directory region */
    FirstDirectorySector = (BPB_NumFATs * FATSz) + BPB_RsvdSecCnt;

    /* Calculate the number of clusters in data region of volume */
    if(0u != BPB_TotSec16)
    {
        CountofClusters = (BPB_TotSec16 - FirstDataSector) / BPB_SecPerClus;
    }
    else
    {
        CountofClusters = (BPB_TotSec32 - FirstDataSector) / BPB_SecPerClus;
    }
 
    // Check that volume is FAT16
    if ((CountofClusters >= DCFS16_FAT12_MAX_CLUSTERS) 
	                     && (CountofClusters < DCFS16_FAT16_MAX_CLUSTERS))
    {
        /* Volume is FAT16 */
        return DCFS16_SUCCESS;
    }
    else
    {
        /* Volume is not FAT16 */
        return DCFS16_FAIL;
    }

}


/*****************************************************************************
 *
 *                             dcfs16_file_exists() 
 *
 *****************************************************************************/
bool_t dcfs16_file_exists(u8_t *name)
{
    /* 
     * Check if a filename pre-exists in root directory.
     */
    return((dcfs16_lookup(name) != DCFS16_NO_FILE) ? 
	                                             DCFS16_TRUE : DCFS16_FALSE);

}


/*****************************************************************************
 *
 *                              dcfs16_open_file() 
 *
 *****************************************************************************/
bool_t dcfs16_open_file(u8_t *name, bool_t attrs)
{
    /*
     * Open a file & allocate file structure. 
     */

    struct Dentry xdata *dirptr;

    dirptr = dcfs16_lookup(name);

    if(dirptr != DCFS16_NO_FILE)
    {
        /* File already exists */
        if(attrs == DCFS16_WRITE)
        {
            /* Zero size on write */
            dirptr->Dsizel = dirptr->Dsizeh = 0;
        }
    }
    else
    {
        if(attrs != DCFS16_WRITE)
        {
            /* Not writing file */
            return DCFS16_FAIL;
        }

        dirptr = dcfs16_create_file(name, 0u);

        if(dirptr == DCFS16_NO_FILE)      
        {    
            /* Unable to create */
            return DCFS16_FAIL;
        }
    }

    /* Fill in file control block from directory information */
    fp.FFirstDirectorySector = wrksec;
    fp.Fdirptr  = dirptr;
    fp.Fattr    = attrs;
    fp.Fnextcls = fp.Ffirstcls = dcfs16_swap_endian_ui(dirptr->Dcluster);
    fp.Fsizel   = dcfs16_swap_endian_ui(dirptr->Dsizel);
    fp.Fsizeh   = dcfs16_swap_endian_ui(dirptr->Dsizeh);
    fp.Fsector  = 0;
    fp.Foffset  = 0;
    fp.Flastcls = 0;

    return DCFS16_SUCCESS;
}


/*****************************************************************************
 *
 *                             dcfs16_close_file() 
 *
 *****************************************************************************/
void dcfs16_close_file(void)
{
    /*
     * Close an open file.
     */

    struct Dentry *dirptr;
    u16_t sizeh, sizel;

    /* Special actions to be taken when writing */
    if(fp.Fattr == DCFS16_WRITE)
    {
        sizel = fp.Fsizel;
        sizeh = fp.Fsizeh;

        /* 
         * If there is a partial last sector, fill it with DOS EOF
         * characters, which also causes it to be written out
         */
        while(fp.Foffset)
        {
            dcfs16_write_byte(DCFS16_DOS_EOF);
        }

        /* 
         * Release remaining clusters in file. If any sectors in the
         * current cluster are used, begin with the next one.    
         */
        dcfs16_release(fp.Fsector 
                       ? dcfs16_swap_endian_ui(dcfs16_get_fat(fp.Fnextcls))
                       : dcfs16_swap_endian_ui(fp.Fnextcls));

        /* Update size entry in file directory */
        dcfs16_read_work(fp.FFirstDirectorySector);
        dirptr            = fp.Fdirptr;
        dirptr->Dcluster  = dcfs16_swap_endian_ui(fp.Ffirstcls);
        dirptr->Dsizel    = dcfs16_swap_endian_ui(sizel);
        dirptr->Dsizeh    = dcfs16_swap_endian_ui(sizeh);

        /* Set directory date of modification here - if you wish */
        wrkchg = DCFS16_TRUE; 
    }

    /* Ensure disk is in sync */
    dcfs16_update_work();

    return;
}


/*****************************************************************************
 *
 *                               dcfs16_delete_file()
 *
 *****************************************************************************/
bool_t dcfs16_delete_file(u8_t *name)
{
    /* 
     * Delete a file from the disk.
     */

    struct Dentry *dirptr;

    if((dirptr = dcfs16_lookup(name)) != DCFS16_NO_FILE)
    {
        *dirptr->Dname = DCFS16_EMPTY;
        wrkchg = DCFS16_TRUE;
        dcfs16_release(dcfs16_swap_endian_ui(dirptr->Dcluster));
        dcfs16_update_work();
        return DCFS16_SUCCESS;
    }

    return DCFS16_FAIL;
}


/*****************************************************************************
 *
 *                               dcfs16_read_byte() 
 *
 *****************************************************************************/
s16_t dcfs16_read_byte(void)
{
    /*
     * Read the next byte from an open file.
     */

    u8_t sector;
    u16_t cluster;

    /* If all data read, return EOF */
    if(!(fp.Fsizel || fp.Fsizeh))
    {
        return DCFS16_EOF;
    }

    /* Decrement file size (32 bit) */
    if((--fp.Fsizel) == 0xFFFFu)
    {
        fp.Fsizeh--;
    }

    /* If no data buffered ... read next sector */
    if((fp.Foffset >= BPB_BytesPerSec) || !fp.Foffset)
    {
        /* If not open for READ, return ERROR */
        if(fp.Fattr != DCFS16_READ)
        {
            return DCFS16_ERROR;
        }

        /* Get next cluster. If past last cluster, return EOF */
        cluster = fp.Fnextcls;
        if((cluster >= DCFS16_LAST_CLUSTER) || (cluster == 0u))
        {
            return DCFS16_EOF;
        }

        /* Read the sector into memory */
        DCFS16_MEDIA_RD_SECTOR((u32_t)((sector = fp.Fsector)
              + ((cluster - 2u) * BPB_SecPerClus) + FirstDataSector), Fbuffer);

        /* Advance sector in cluster, if past end, get next cluster number */
        if(++sector >= BPB_SecPerClus)
        {
            fp.Fnextcls = dcfs16_get_fat(cluster);
            sector = 0u;
        }

        /* Update file control block information */
        fp.Flastcls = cluster;
        fp.Fsector  = sector;
        fp.Foffset  = 0u;
    }

    /* Read character, and advance circular buffer pointer */
    return(Fbuffer[fp.Foffset++]);
}


/*****************************************************************************
 *
 *                             dcfs16_write_byte()  
 *
 *****************************************************************************/
bool_t dcfs16_write_byte(u8_t c)
{
    /*
     * Write a byte to an open file.
     */

    u8_t sector;
    u16_t cluster;

    /* Test for writeable file */
    if(fp.Fattr != DCFS16_WRITE)
    {
        return DCFS16_FAIL;
    }

    /* Advance file size */
    if(!++fp.Fsizel)
    {
        ++fp.Fsizeh;
    }

    /* Write character to buffer */
    Fbuffer[fp.Foffset++] = c;

    /* If buffer is full, write it */
    if(fp.Foffset >= BPB_BytesPerSec)
    {
        /* If no sector allocated, allocate one */
        if(!fp.Fnextcls)
        {
            fp.Ffirstcls = fp.Fnextcls = dcfs16_allocate(0u);
        }
        else if(fp.Fnextcls >= DCFS16_LAST_CLUSTER)
        {
            fp.Fnextcls = dcfs16_allocate(fp.Flastcls);
        }
        else
        {
            /* MISRA compliance */
        }

        cluster = fp.Fnextcls;

        if(!cluster)
        {
            return DCFS16_FAIL;
        }

        /* Write the data to the drive */    
        DCFS16_MEDIA_WR_SECTOR((u32_t)((sector = fp.Fsector)
              + ((cluster - 2u) * BPB_SecPerClus) + FirstDataSector), Fbuffer);

        /* Advance to next sector in cluster */
        if(++sector >= BPB_SecPerClus)
        {
            fp.Fnextcls = dcfs16_get_fat(cluster);
            sector = 0;
        }

        /* Update file control block information */
        fp.Flastcls = cluster;
        fp.Fsector  = sector;
        fp.Foffset  = 0;
    }

    return DCFS16_SUCCESS;
}


/*****************************************************************************
 *
 *                        Private Function Implementation
 *
 *****************************************************************************/

/*****************************************************************************
 *
 *                               dcfs16_allocate()     
 *
 *****************************************************************************/
static u16_t dcfs16_allocate(u16_t cluster)
{
    /*
     * Allocate a free cluster on disk, and cross connect FAT if necessary
     * mark cluster as used & end of file. 
     */

    u16_t begin, end, i;

    /* 
     * Calculate start and end clusters for search.
     * If we have a "FAT" sector loaded, begin searching from there,
     * To attempt to keep allocated BPB_TotSec16 in same "FAT" sector 
     */
    i = wrksec - 1u;

    begin = ((i) && (i <= BPB_FATSz16)) ?
                           ((i * (DCFS16_SECTOR_SIZE * 2u)) / 3u) + 1u : 2u;
    end = CountofClusters;

    do
    {
        for(i = begin; i < end; ++i)
        {
            if(!dcfs16_get_fat(i))          /* Cluster free ? */
            {
                dcfs16_set_fat(i, DCFS16_END_CLUSTER);
                if(0u != cluster)
                {
                    dcfs16_set_fat(cluster, i);
                }
                return i;
            }
        }

        /* Didn't find, reset to start cluster, and try again */
        end = begin;
        begin = 2u;
    } while(end != 2u);

    return DCFS16_FAIL;
}


/*****************************************************************************
 *
 *                                dcfs16_release()    
 *
 *****************************************************************************/
static void dcfs16_release(u16_t cluster)
{
    /* 
     * Release a chain of allocated clusters. 
     */

    u16_t i;

    while((cluster != 0u) && (cluster < DCFS16_BAD_CLUSTER))
    {
        i = dcfs16_get_fat(cluster);
        dcfs16_set_fat(cluster, 0u);
        cluster = i;
    }

    return;
}


/*****************************************************************************
 *
 *                                dcfs16_get_fat()   
 *
 *****************************************************************************/
static u16_t dcfs16_get_fat(u16_t cluster)
{
    /* 
     * Get a FAT entry for a given cluster number.
     */

    cluster *= 2u;
    
    dcfs16_read_work((cluster / BPB_BytesPerSec) + 1u);

    return(wrkbuff[cluster % BPB_BytesPerSec] 
                         + (wrkbuff[(cluster + 1u) % BPB_BytesPerSec] << 8));
}


/*****************************************************************************
 *
 *                                dcfs16_set_fat()
 *
 *****************************************************************************/
static void dcfs16_set_fat(u16_t cluster, u16_t value)
{
    /* 
     * Set a FAT entry for a given cluster number.
     */

    cluster *= 2u;

    dcfs16_read_work((cluster / BPB_BytesPerSec) + 1u);

    wrkbuff[cluster % BPB_BytesPerSec] = (value & 0xFFu);
    wrkbuff[(cluster + 1u) % BPB_BytesPerSec] = (u8_t)(value >> 8);

    wrkchg = DCFS16_TRUE;

    return;
}


/*****************************************************************************
 *
 *                                 dcfs16_lookup()  
 *
 *****************************************************************************/
static struct Dentry xdata *dcfs16_lookup(u8_t *file)
{
    /*
     * Lookup a directory entry & return a pointer to it.
     */

    bool_t name_found = 0u;
    u16_t i, j, k;
    u8_t m;
    u8_t filename_buffer[12];

    dcfs16_parse_filename(filename_buffer, file);

    j = BPB_BytesPerSec;
    k = FirstDirectorySector;

    /*
     * Step through directory entries looking for specified filename.
     */
    for(i = 0u; i < BPB_RootEntCnt; i++)
    {
        /*
         * If end of sector reached, load the next one.
         */
        if(j >= BPB_BytesPerSec)
        {
            dcfs16_read_work(k++);
            j = 0u;
        }

        /*
         * Compare directory entry filename with specified filename.
         * if(!memcmp(wrkbuff + j, filename_buffer, 11))
         * {
         *     return (struct Dentry xdata *)(wrkbuff + j);
         * }
         */                 
        for (m = 0u, name_found = 1u; m < 12u; m++)
        {
            if (*(wrkbuff + j + m) != filename_buffer[m])
            {
                name_found = 0u;
            }
        }

        if (name_found)
        {
            return (struct Dentry xdata *)(wrkbuff + j);
        }

        j += sizeof(struct Dentry);
    }

    return DCFS16_NO_FILE;
}


/*****************************************************************************
 *
 *                             dcfs16_create_file()  
 *
 *****************************************************************************/
static struct Dentry xdata *dcfs16_create_file(u8_t *file, u8_t attrs)
{
    /* 
     * Create a file with the specified name.
     * NOTE: Does NOT check for duplicates. Its assumed that "dcfs16_lookup()"
     * has been called first, to verify that the file does not already
     * exist on the disk.                                                  
     */

    u16_t i, j, k;
    u8_t fbuffer[12];
    struct Dentry xdata *dirptr;

    dcfs16_parse_filename(fbuffer, file);

    j = BPB_BytesPerSec;
    k = FirstDirectorySector;

    for(i = 0; i < BPB_RootEntCnt; ++i)
    {
        if(j >= BPB_BytesPerSec)
        {
            dcfs16_read_work(k++);
            j = 0;
        }

		/* Is this a free directory entry? */
        if((*(dirptr = (struct Dentry xdata *)(wrkbuff + j))->Dname
	                                == DCFS16_EMPTY) || (!*dirptr->Dname))
        {
            u8_t count = sizeof(struct Dentry);
            u8_t xdata * cntptr = (u8_t xdata *)dirptr;

            /* 
             * Zero-out the directory entry.
             * memset(dirptr, (char)0, (s16_t)sizeof(struct Dentry)); 
             */
            while(count--)
            {
                *cntptr++ = 0u;
            }

            /* 
             * Copy new filename into directory entry.
             * strcpy(dirptr, fbuffer); 
             */
            count = 0;
            cntptr = (u8_t xdata *)dirptr;
            while((*cntptr++ = fbuffer[count++]) != '\0')
            {
                continue;
            }

            /* Set new file attributes */
            dirptr->Dattr = attrs;

            /* Set directory date of creation here - if you wish */

            wrkchg = DCFS16_TRUE;

            return((struct Dentry xdata *)(wrkbuff + j));
        }

        j += sizeof(struct Dentry);
    }

    return DCFS16_NO_FILE;
}


/*****************************************************************************
 *
 *                            dcfs16_parse_filename() 
 *
 *****************************************************************************/
static void dcfs16_parse_filename(u8_t buffer[], u8_t *name)
{
    /* 
     * Parse a filename into directory format (12 characters, space padded)
     */

    u16_t i = 0u;
 
    while(i < 8u)
    {
        buffer[i++] = (*name && (*name != '.')) ? toupper(*name++) : ' ';
    }

    if(*name == '.')
    {
        name++;
    }

    while(i < 11u)
    {
        buffer[i++] = *name ? toupper(*name++) : ' ';
    }

    buffer[i] = 0;

    return;
}


/*****************************************************************************
 *
 *                               dcfs16_read_work()   
 *
 *****************************************************************************/
static void dcfs16_read_work(u16_t sector)
{
    /* 
     * Read a work sector into memory if it is not already in memory. 
     */

    if(wrksec != sector)
    {
        dcfs16_update_work();
        wrksec = sector;
        DCFS16_MEDIA_RD_SECTOR((u32_t)wrksec, wrkbuff);
    }

    return;
}


/*****************************************************************************
 *
 *                              dcfs16_update_work()
 *
 *****************************************************************************/
static void dcfs16_update_work(void)
{
    /* 
     * Write the work sector if it has been marked as modified.
     */

    if(wrkchg)
    {
        if ((BPB_NumFATs > 1) && (wrksec < FirstDirectorySector))
        {
            /*
             * Volume has 2 FATs and we are writing to first one,
             * so duplicate result to second FAT.
             */
            DCFS16_MEDIA_WR_SECTOR((u32_t)(wrksec + BPB_FATSz16), wrkbuff);
        }

        DCFS16_MEDIA_WR_SECTOR((u32_t)(wrksec), wrkbuff);
        wrkchg = DCFS16_FALSE;
    }

    return;
}


/*****************************************************************************
 *
 *                            dcfs16_swap_endian_ui()
 *
 *****************************************************************************/
static u16_t dcfs16_swap_endian_ui(u16_t fromVal) 
{
    /*
     * Convert Big Endian <-> Little Endian for unsigned integer.
     */

    u16_t toVal;
   
    ((u8_t *) &toVal)[0] = ((u8_t *) &fromVal)[1];
    ((u8_t *) &toVal)[1] = ((u8_t *) &fromVal)[0];

    return(toVal);
}


/*****************************************************************************
 *
 *                            dcfs16_swap_endian_ul()
 *
 *****************************************************************************/
static u32_t dcfs16_swap_endian_ul(u32_t fromVal) 
{
    /* 
     * Convert Big Endian <-> Little Endian for unsigned long integer.
     */

    u32_t toVal;
   
    ((u8_t *) &toVal)[0] = ((u8_t *) &fromVal)[3];
    ((u8_t *) &toVal)[1] = ((u8_t *) &fromVal)[2];
    ((u8_t *) &toVal)[2] = ((u8_t *) &fromVal)[1];
    ((u8_t *) &toVal)[3] = ((u8_t *) &fromVal)[0];

    return(toVal);
}

/** @} */

