/**
 * \addtogroup Dcfs16
 * @{
 */

/*****************************************************************************/
/**
 * \file Dcfs16.h
 * FAT16 DOS root directory file system for the 8051 and Keil C51.
 * \author Copyright (c) 2005, Murray R. Van Luyn. <vanluynm@iinet.net.au>
 * \version 0.1
 * \date 24-06-05
 */
/*****************************************************************************/

/*****************************************************************************
 *
 *     THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 *     OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *     ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 *     DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *     DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 *     GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *     INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *     WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *     NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *****************************************************************************/

#ifndef DCFS16_H
#define DCFS16_H

#include "REG51.h"
#include "types.h"
#include <ctype.h>

/*****************************************************************************
 *
 *                               Macro Definitions
 *
 *****************************************************************************/


/*****************************************************************************
 *
 *                               Defined Constants
 *
 *****************************************************************************/

/**
 * \name Sector level media specific function defines.
 * @{
 *
 * Sector level media initialisation function defines are used to statically 
 * link file system code to media specific sector level access code.
 */

/** 
 * Sector level media access function header define.
 * Include here the header for sector level access function code.
 * \hideinitializer
 */
#include "CFlash.h"

/** 
 * Sector level media initialisation function define.
 * Set \b DCFS16_MEDIA_INIT to the name of the sector level function that
 * initialises the storage media.
 * \hideinitializer
 */
#define DCFS16_MEDIA_INIT             cflash_init

/** 
 * Sector level media availability test function define.
 * Set \b DCFS16_MEDIA_DOCKED to the name of the sector level function that
 * determines whether storage media is available.
 * \hideinitializer
 */
#define DCFS16_MEDIA_DOCKED           CFLASH_DOCKED

/** 
 * Sector level media read function define.
 * Set \b DCFS16_MEDIA_RD_SECTOR to the name of the sector level function that
 * reads from the storage media.
 * \hideinitializer
 */
#define DCFS16_MEDIA_RD_SECTOR        cflash_rd_sector

/** 
 * Sector level media write function define.
 * Set \b DCFS16_MEDIA_WR_SECTOR to the name of the sector level function that
 * writes to the storage media.
 * \hideinitializer
 */
#define DCFS16_MEDIA_WR_SECTOR        cflash_wr_sector

/** @} */


/** 
 * File system function return value.
 * \b DCFS16_SUCCESS is returned by drive mounting, and file open, write and
 * delete functions to indicate operation success.
 * \hideinitializer
 */
#define DCFS16_SUCCESS                1

/** 
 * File system function return value.
 * \b DCFS16_FAIL is returned by drive mounting, and file open, write and
 * delete functions to indicate operation failure.
 * \hideinitializer
 */
#define DCFS16_FAIL                   0

/** 
 * File system function return value.
 * \b DCFS16_EOF is returned by a file read operation to indicate that the 
 * end of the file has been reached.
 * \hideinitializer
 */
#define DCFS16_EOF                   -1

/** 
 * File system function return value.
 * \b DCFS16_ERROR is returned by a file read operation to indicate that the 
 * a file is not open for reading.
 * \hideinitializer
 */
#define DCFS16_ERROR                 -2

/** 
 * File system function return value.
 * \b DCFS16_TRUE is returned by a directory entry existence check operation
 * to indicate that a filename exists.
 * \hideinitializer
 */
#define DCFS16_TRUE                   1

/** 
 * File system function return value.
 * \b DCFS16_FALSE is returned by a directory entry existence check operation
 * to indicate that the a filename does not exist.
 * \hideinitializer
 */
#define DCFS16_FALSE                  0

/** 
 * File system function parameter.
 * \b DCFS16_READ is passed to a file open operation to specify read access.
 * \hideinitializer
 */
#define DCFS16_READ                   0

/** 
 * File system function parameter.
 * \b DCFS16_WRITE is passed to a file open operation to specify write access.
 * \hideinitializer
 */
#define DCFS16_WRITE                  1


/* 
 * Miscellaneous fixed parameters.
 */
#define DCFS16_SECTOR_SIZE          512u   /* Size of a disk sector         */
#define DCFS16_BPB_SECTOR             0u   /* BIOS Parameter block sector   */
#define DCFS16_EMPTY               0xE5u   /* Signals empty directory       */
#define DCFS16_DOS_EOF             0x1Au   /* DOS end of file marker        */
#define DCFS16_FAT12_MAX_CLUSTERS  4085u   /* Most clusters FAT12 can have  */
#define DCFS16_FAT16_MAX_CLUSTERS 65525u   /* Most clusters FAT16 can have  */
#define DCFS16_BAD_CLUSTER       0xFFFFu   /* Bad cluster marker            */
#define DCFS16_LAST_CLUSTER      0xFFF8u   /* Last cluster marker           */
#define DCFS16_END_CLUSTER       0xFFFFu   /* End cluster marker            */
#define DCFS16_NO_FILE                0u   /* File create/ lookup fail flag */


/*****************************************************************************
 *
 *                           Public Function Prototypes
 *
 *****************************************************************************/

/** 
 * File system and media module initialisation function.
 * Mount a disk drive and set up control information
 * THIS FUNCTION MUST BE CALLED BEFORE ACCESSING ANY FILES.
 * \param -
 * \return A status \b bool_t. Either #DCFS16_SUCCESS or #DCFS16_FAIL
 */
extern bool_t dcfs16_mount_drive(void);

/** 
 * Existing directory entry lookup function.
 * \param name An \b unsigned \b u8_t \b * to name of file to look for.
 * \return A status \b bool_t. Either #DCFS16_TRUE or #DCFS16_FALSE
 */
extern bool_t dcfs16_file_exists(u8_t *name);

/** 
 * Open a file for read or write access.
 * \param name An \b unsigned \b u8_t \b * to name of file to open.
 * \param attrs A \b bool_t. Either #DCFS16_READ or #DCFS16_WRITE
 * \return A status \b bool_t. Either #DCFS16_SUCCESS or #DCFS16_FAIL
 */
extern bool_t dcfs16_open_file(u8_t *name, bool_t attrs);

/** 
 * Close a file opened for read or write access.
 * \param -
 * \return -
 */
extern void dcfs16_close_file(void);

/** 
 * Delete a named file.
 * \param name An \b unsigned \b u8_t \b * to name of file to delete.
 * \return A status \b bool_t. Either #DCFS16_SUCCESS or #DCFS16_FAIL
 */
extern bool_t dcfs16_delete_file(u8_t *name);

/** 
 * Read the next byte from an open file.
 * \param -
 * \return A \b signed \b s16_t containing the value of the byte read,
 * #DCFS16_ERROR or #DCFS16_EOF.
 * \warning This function returns a s16_t and not an unsigned u8_t.
 */
extern s16_t dcfs16_read_byte(void);

/** 
 * Write a byte to an open file.
 * \param c An \b unsigned \b u8_t value to write.
 * \return A status \b bool_t. Either #DCFS16_SUCCESS or #DCFS16_FAIL
 */
extern bool_t  dcfs16_write_byte(u8_t c);

#endif

/** @} */

